// $Id: Call.java,v 1.1 2005/01/10 20:04:01 jwalter Exp $
package net.jsa;

/**
 * Class representing a call between current user and another user or a pstn
 * 
 * 
 * @version $Revision: 1.1 $ $Date: 2005/01/10 20:04:01 $
 * @author Joakim Waltersson
 */
public class Call extends CommunicationEntity {
    /* Incoming pstn call */
    public static final int TYPE_INCOMING_PSTN = 0;
    
    /* Outgoing pstn call */
    public static final int TYPE_OUTGOING_PSTN = 1;
    
    /* Incoming p2p call */ 
    public static final int TYPE_INCOMING_P2P = 2;
    
    /* Outgoing p2p call */
    public static final int TYPE_OUTGOING_P2P = 3;
    
    
    /* Call was never placed */
    public static final int STATUS_UNPLACED = 0;
    
    /* Call is currenty being routed */
    public static final int STATUS_ROUTING = 1;
    
    /* With the pstn there is possibility that before the call is actually 
     * established, the early media is being played. For example it can be a
     * calling tone or it can be some waiting message (all operators are busy,
     * hold on for a sec) etc. */
    public static final int STATUS_EARLYMEDIA = 2;
    
    /* Call failed. Try to get FAILUREREASON for more information. */
    public static final int STATUS_FAILED = 3;
    
    public static final int STATUS_RINGING = 4;
    public static final int STATUS_INPROGRESS = 5;
    public static final int STATUS_ONHOLD = 6;
    public static final int STATUS_FINISHED = 7;
    public static final int STATUS_MISSED = 8;
    public static final int STATUS_REFUSED = 9;
    public static final int STATUS_BUSY = 10;
    public static final int STATUS_CANCELLED = 11;
    
    public static final int FAILUREREASON_NOVALIDPSTN = 1;
    public static final int FAILUREREASON_NOSUCHNUMBER = 2;
    public static final int FAILUREREASON_USEROFFLINE = 3;
    public static final int FAILUREREASON_NOPROXY = 4;
    public static final int FAILUREREASON_SESSIONTERMINATED = 5;
    public static final int FAILUREREASON_NOCODEC = 6;
    public static final int FAILUREREASON_SOUNDERROR = 7;
    public static final int FAILUREREASON_REMOTESOUNDERROR = 8;
    public static final int FAILUREREASON_BLOCKED = 9;
    public static final int FAILUREREASON_NOTFRIEND = 10;
    public static final int FAILUREREASON_NOTAUTHORISED = 11;
    
    private int type;
    private int status;
    private String subject;
    private String pstnNumber;
    private long duration;
    private String pstnStatus;

    /**
     * Get the status of this call
     * 
     * @return Returns the status.
     */
    public int getStatus() {
        return status;
    }
    /**
     * @return Returns the subject.
     */
    public String getSubject() {
        return subject;
    }
    /**
     * @return Returns the type.
     */
    public int getType() {
        return type;
    }
    
}
