package net.jsa.connector;

import java.util.Date;
import java.util.Observable;

/**
 * Copyright (C) 2004  B. Lamot
 ***********************************************************
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 ************************************************************
 *
 * A basic implementation all Messengers need, the listener method are implemented.
 * The sendMessage method should be overridden.
 */
public abstract class AbstractSkypeAPIConnector extends Observable {
		
	/**
	 * To be called by the messengers that extend this class.
	 * This will fire all the listeners.
	 * @param message the message the listeners have to be fired with.
	 */
	protected final void onMessageReceived(final String message){
        setChanged();
        notifyObservers(message);
	}
	private String translate(String s) {
		String lots = "LASTONLINETIMESTAMP";
		int dateIdx = s.indexOf(lots);
		if (dateIdx == -1) {
			return s;
		}
		try {
			dateIdx += lots.length() + 1;
			long dateLong = Long.parseLong(s.substring(dateIdx));
			dateLong *= 1000L;
			return s.substring(0, dateIdx) + new Date(dateLong).toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return s;
	}
	/**
	 * Messengers should override this method with their sendMessage implementation, dependant on the platform.
	 * @param message the message to send.
	 * @return false if message could not be send.
	 */
	abstract public boolean sendMessage(String message);
	
	/**
	 * Used to initialize the platform specific initialisation.
	 * @return false if initialisation went wrong.
	 */
	abstract public boolean initialize();
	
	/**
	 * Terminate this messenger and let it clean-up it's trash (jni trash or sockets and stuff).
	 *
	 */
	abstract public void destroy();
	
}
