package net.jsa.connector.tcp;

import java.net.ServerSocket;
import java.net.Socket;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;

import net.jsa.connector.AbstractSkypeAPIConnector;
import net.jsa.connector.windows.SkypeAPIConnector;

/**
* Copyright (C) 2004  B. Lamot
 ***********************************************************
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 ************************************************************
 *
 * This class is a executable class which starts a tcp/ip Skype API Service.
 * TODO: read the port number from a property file
 * TODO: use the messengerfactory
 */
public class ServerMain implements Observer {

	private int port = 9998;
	private ServerSocket socket = null;
	private AbstractSkypeAPIConnector messenger = new SkypeAPIConnector();
	private HashSet clients = new HashSet(5);

	
	/**
	 * Constructor.
	 * Add this instance as a listener to the messenger.
	 * Initialize the messenger.
	 */
	public ServerMain() {
		messenger.addObserver(this);
		messenger.initialize();
	}

	/**
	 * Open the server socket.
	 * Wait for a client
	 * When a client connects start it's thread.
	 * and wait again.
	 */
	public void startServer() {
		try {
			socket = new ServerSocket(port);
			while (socket != null) {
				Socket clientsocket = socket.accept();
				System.out.println("ServerMain.dostuff(): new client");
				ServerThread st = new ServerThread(clientsocket, messenger);
				clients.add(st);
				st.start();
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}

	}

	/**
	 * The entrance of the server program.
	 * @param args none.
	 */
	public static void main(String[] args) {
		new ServerMain().startServer();
	}

	/**
	 * This method is an implementation of the AbstractMessenger.
	 * When a message comes from the Skype API send it to all clients.
	 */
	public void onMessageReceived(String message) {
		System.out.println("ServerMain.onMessage(); message=" + message);
		Iterator it = clients.iterator();
		ServerThread st = null;
		while (it.hasNext()) {
			st = (ServerThread) it.next();
			if (!st.sendSkypeMessage(message)) {
				clients.remove(st);
			}
		}
	}

    /* (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable arg0, Object arg1) {
        onMessageReceived(arg1.toString());
    }
}
