package net.jsa.connector.tcp;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.Socket;

import net.jsa.connector.AbstractSkypeAPIConnector;

/**
* Copyright (C) 2004  B. Lamot
 ***********************************************************
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 ************************************************************
 *
 *	This class is to be initiated from the ServerMain class.
 *	For each client that connects to the ServerMain service a thread like this will be started.
 *
 */
public class ServerThread extends Thread {

	private Socket socket = null;
	private AbstractSkypeAPIConnector messenger = null;

	/**
	 * Constructor.
	 * @param newSocket the socket to listen and write to.
	 * @param newMessenger the messenger to listen and write to.
	 */
	public ServerThread(Socket newSocket, AbstractSkypeAPIConnector newMessenger) {
		System.out.println("ServerThread.ServerThread();");
		socket = newSocket;
		messenger = newMessenger;
	}

	/**
	 * The Service thread main loop.
	 * As long as the socket is valid try to read commands from it.
	 * Each received command is sent to the skypeAPI.
	 */
	public void run() {
		System.out.println("ServerThread.run()");
		BufferedReader reader = null;
		try {
			reader =
				new BufferedReader(
					new InputStreamReader(socket.getInputStream(), "UTF-8"));
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		String message = null;
		while (socket != null) {
			try {
				message = reader.readLine();
				System.out.println("ServerThread.run(); message received:"+message);
				if (message == null) {
					socket.close();
					socket=null;
				} else {
					messenger.sendMessage(message);
				}					
			} catch (IOException e1) {
				socket=null;
				e1.printStackTrace();
			}
		}
	}

	/**
	 * Call this message when a msg from the SkypeAPI needs to be sent to the tcp/ip clients.
	 * @param message the skypeAPI message.
	 * @return false if the message couldn't be sent.
	 */
	public boolean sendSkypeMessage(String message) {
		System.out.println("ServerThread.sendSkypeMessage()");
		if (socket == null) {
			return false;
		}
		try {
			BufferedWriter writer =
				new BufferedWriter(
					new OutputStreamWriter(socket.getOutputStream(), "UTF-8"));
			writer.write(message+"\n");
			writer.flush();
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}
}
