package net.jsa.connector.tcp;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.Socket;

import net.jsa.connector.AbstractSkypeAPIConnector;

/**
 * Copyright (C) 2004  B. Lamot
 ***********************************************************
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 ************************************************************
 *
 * To get from java client to the Skype API there needs to be an implementation of a Messenger.
 * This messenger connects to a Skype API using tcp/ip (so skype can be on a remote pc).
 *  
 * If you look at this messenger in a tcp-ip sence this is the client connecting to a server.
 */
public class SkypeAPIConnector extends AbstractSkypeAPIConnector {
	
	private Socket socket = null;
	private String addr = "127.0.0.1";
	private int port = 9998;
	private static SkypeAPIConnector _instance = null;
	private boolean destroyed = false;
	
	/**
	 * Initialize this messenger.
	 * It opens a tcp/ip socket connection and starts a reading thread.
	 * @return false when the connection could not be established.
	 */
	public boolean initialize() {
		System.out.println("(tcp)Messenger.initialize()");
		_instance = this;
		//Try to open a socket.
		try {
			socket = new Socket(addr,port);
		} catch (Exception e) {
			e.printStackTrace();
			socket = null;
			return false;
		}
		//Start a reading thread.
		new Thread(new Runnable() {
			public void run() {
				BufferedReader reader = null;		
				try {
					reader = new BufferedReader(new InputStreamReader(socket.getInputStream(),"UTF-8"));
				} catch (UnsupportedEncodingException e) {
					
					e.printStackTrace();
				} catch (IOException e) {
					
					e.printStackTrace();
				}  
				String message = null;
				while (socket != null || !destroyed) {				
					try {
						message = reader.readLine();
						_instance.onMessageReceived(message);
					} catch (IOException e1) {		
						destroy();				
					}
				}	  
			}
		  }).start();
		return true;
	}
	
	/**
	 * This method send the skype command to the net.lamot.java.jskype.tcp.ServerThread.
	 *	@param message the skype command to send.
	 *	@return false if message couldnt be send. 
	 */ 
	public boolean sendMessage(String message){ 
		if (socket==null) {
			System.out.println("Messenger.sendMessage(); couldn't send message socket==null");
			return false;
		}
		try {
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream(),"UTF-8"));
			writer.write(message+"\n");
			writer.flush();
			System.out.println("Messenger.sendMessage(); message sent");
		} catch (Exception e) {
			socket = null;
			e.printStackTrace();
		}
		return true;
	}

	/**
	 * Call this method when the messenger is no longer needed.
	 */
	public void destroy() {
		destroyed = true;
		try {
			socket.close();
		} catch (Exception e) {
			//An error isn't really intresting cause we want to throw the thing away anyways.
		}	
		socket = null;		
	}
		

}
